from .base import File
from .audio.mp3 import Mp3File
from .documents.html import HtmlFile
from .images.jpeg import JpegFile
from .video.mkv import MkvFile
from .video.mp4 import Mp4File
from .video.webm import WebmFile
